/*
   Name: Password Validator for glFtpD
   Version: 2.0
   Author: Zio (v1.0), HoE (v2.0)
   Last updated: someday in 2004
   Info:
   A slight modification of someones passchk.c.
   The author is anonymous afaik. No info in the original passchk.c.
   Validates a users password from /glftpd/etc/passwd file.
   Can be used for inviting people to an IRC channel based on their
   account on the site.

   And "chmod +s /glftpd/bin/passchk" since usually only root can access 
   the passwd file.

   V2.0 changes :
   - updated for v2.0 password scheme, uses multi hmac-sha1 hash
   - removed chroot
   - TODO : fix memory leaks...
   
   Outputs a "1" if match and a "0" if no match.
*/

#include <pwd.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <paths.h>
#include <openssl/evp.h>
#include <openssl/hmac.h>


//see http://lists.jammed.com/secprog/2002/11/0008.html for info

#define HLEN (20)		/*Using SHA-1 */

int pbkdf2(const unsigned char *pw, unsigned int pwlen, char *salt,
	   unsigned long long saltlen, unsigned int ic,
	   unsigned char *dk, unsigned long long dklen)
{
    unsigned long l, r, i, j;
    unsigned char txt[4], hash[HLEN * 2], tmp[HLEN], *p =
	dk, *lhix, *hix, *swap;
    short k;
    int outlen;

    if (dklen > ((((unsigned long long) 1) << 32) - 1) * HLEN) {
	/* TODO: Call an error handler. */
	return 1;
    }
    l = dklen / HLEN;
    r = dklen % HLEN;

    for (i = 1; i <= l; i++) {
	sprintf(txt, "%04u", (unsigned int) i);
	HMAC(EVP_sha1(), pw, pwlen, txt, 4, hash, &outlen);
	lhix = hash;
	hix = hash + HLEN;
	for (k = 0; k < HLEN; k++) {
	    tmp[k] = hash[k];
	}
	for (j = 1; j < ic; j++) {
	    HMAC(EVP_sha1(), pw, pwlen, lhix, HLEN, hix, &outlen);
	    for (k = 0; k < HLEN; k++) {
		tmp[k] ^= hix[k];
	    }
	    swap = hix;
	    hix = lhix;
	    lhix = swap;
	}
	for (k = 0; k < HLEN; k++) {
	    *p++ = tmp[k];
	}
    }
    if (r) {
	sprintf(txt, "%04u", (unsigned int) i);
	HMAC(EVP_sha1(), pw, pwlen, txt, 4, hash, &outlen);
	lhix = hash;
	hix = hash + HLEN;
	for (k = 0; k < HLEN; k++) {
	    tmp[k] = hash[k];
	}
	for (j = 1; j < ic; j++) {
	    HMAC(EVP_sha1(), pw, pwlen, lhix, HLEN, hix, &outlen);
	    for (k = 0; k < HLEN; k++) {
		tmp[k] ^= hix[k];
	    }
	    swap = hix;
	    hix = lhix;
	    lhix = swap;
	}
	for (k = 0; k < r; k++) {
	    *p++ = tmp[k];
	}
    }
    return 0;
}

int pw_encrypt(const unsigned char *pwd, char *digest, char *seed) {
        unsigned char md[SHA_DIGEST_LENGTH];
        int mdlen=SHA_DIGEST_LENGTH, i;

        //see pbe.c for info, 100x multihash
        pbkdf2(pwd, strlen(pwd), seed, strlen(seed), 100, md, SHA_DIGEST_LENGTH);

        for(i = 0; i < mdlen; i++) {
                sprintf(digest, "%02x", md[i]);
                digest += 2;
        }
        //fix the last /0 !!!
        *digest='\0';
        return 1;
}


int main(int argc, char *argv[])
{
    char passwd[BUFSIZ];
    char xpasswd[SHA_DIGEST_LENGTH * 2 + 1];
    char *salt;
    char user[BUFSIZ];
    struct passwd *pw;

    FILE *f;

    if (argc < 3) {
	printf("glFtpD Password Validator v0.5 (C) Zio 2000\n");
	printf("(The passwd file must be located in /glftpd/etc/)\n");
	printf("Usage: %s <user> <password>\n", argv[0]);
	exit(0);
    }
    strncpy(user, argv[1], sizeof(user));
    strncpy(passwd, argv[2], sizeof(passwd));

    if (!(f = fopen("/glftpd/etc/passwd", "r"))) {
	fprintf(stderr, "unable to open passwd file");
	puts("0");
	return 0;
    }

    do {
	pw = fgetpwent(f);
	if (pw) {
	    if (strcmp(pw->pw_name, user) == 0) {
		salt = pw->pw_gecos;
		bzero(xpasswd, sizeof(xpasswd));
		//printf("%s\n", pw->pw_gecos);
		pw_encrypt(passwd,xpasswd,salt);
		//printf("%s\n", pw->pw_passwd);
		//printf("%s\n", xpasswd);
		if ((pw == NULL) || *pw->pw_passwd == '\0'
		    || strcmp(xpasswd, pw->pw_passwd)) {
		    printf("0\n");
		    exit(0);
		}
		printf("1\n");
		exit(0);

	    }
	}
    } while (pw);

    fclose(f);
    printf("0\n");
    exit(0);
}
